
# q-expansion of newform 529.2.c.e, downloaded from the LMFDB on 18 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 529
weight = 2
poly_data = [1, -1, 1, -1, 1, -1, 1, -1, 1, -1, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[5, [0, 0, 0, 0, 1, 0, 0, 0, 0, 0]]]
aps_data = [[1, 0, 1, -1, 1, -1, 1, 0, 1, 0], [-1, 1, 0, 0, 0, 0, 0, 0, -1, 1], [0, 1, 1, 1, 0, 0, 0, 1, -1, 0], [0, -1, 0, 0, 1, -2, 1, 0, 0, -1], [0, -1, 1, 0, 0, -1, 0, -3, 0, -1], [1, -1, 0, 1, 1, 0, 1, 0, 1, 1], [1, 0, 1, -4, 1, 0, 1, 0, 2, -2], [0, 2, 0, 0, -2, 0, 0, -2, 2, -2], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0], [2, -3, 4, -4, 4, -3, 2, 0, 4, -4], [-2, 0, -2, 0, 0, 1, -2, 2, -1, 0], [-3, 0, 1, 1, -1, -1, 0, 3, 0, 2], [-3, 0, 1, 0, -3, 0, 0, 1, -1, 0], [-2, -1, 0, 0, 1, -1, 0, 0, 1, 2], [-3, 0, -3, 1, -2, 0, 0, 2, -1, 3], [0, 6, -5, 4, -7, 4, -7, 4, -5, 6], [0, 0, 0, -3, 0, 2, -3, 2, 0, -3], [-3, -1, -3, 0, 3, 1, 2, -2, -1, -3], [-2, 7, -1, 3, -6, 3, -1, 7, -2, 0], [-3, 3, -1, 1, -3, 1, -1, 3, -3, 0], [2, -3, 4, -4, 3, -2, 0, -4, -1, -4], [0, 0, 0, -6, -2, -3, 4, -3, -2, -6], [0, 1, -3, 4, -4, 3, -1, 0, 0, -1], [-1, 5, 3, 4, -8, 8, -4, -3, -5, 1], [2, -7, 1, -7, 2, 0, 7, -5, 5, -7], [-3, 5, -7, -1, 1, 7, -5, 3, 0, 8], [-3, 5, -5, 3, 0, -5, -2, -1, -2, -5], [3, -4, 3, 0, -2, -3, 4, -4, 3, 2], [-1, -2, -8, 6, -8, -2, -1, 0, -1, 1], [1, 1, 1, -1, 0, -1, 1, 1, 1, 0], [-7, 2, -2, 7, 0, 8, -2, -3, -2, 8], [0, -3, 1, -3, -3, 6, -3, -3, 1, -3], [-3, 0, -4, 2, 2, 4, -4, -2, -2, 4], [1, 0, 7, 7, -1, 2, -2, 1, -7, -7], [-4, 4, -4, 4, 0, 4, 3, -2, 3, 4], [-7, 7, 0, -1, -4, -2, 0, -2, -4, -1], [3, -8, -2, 2, 8, -3, 0, -2, 0, -2], [3, -2, -1, 8, 4, 8, -1, -2, 3, 0], [-5, 4, -2, 3, -2, 4, -5, 0, -5, 5], [3, 1, -1, -3, 0, 1, -6, 4, -6, 1], [-1, 10, 2, 10, -1, 0, -5, 4, -4, 5], [-3, -7, -4, 5, -4, 4, -5, 4, 7, 3], [0, -5, -3, -2, 7, -13, 7, -2, -3, -5], [-1, -2, -2, 7, -7, 2, 2, 1, 0, 0], [-1, 1, 0, 3, 3, -5, -2, -5, 3, 3], [-5, 4, -5, 0, -7, 5, -5, 5, -5, 7], [-3, 2, -1, 1, -2, 3, 0, 8, -6, 8], [0, 6, -12, 9, 0, 8, 0, 9, -12, 6], [-11, 7, -10, -5, -5, 5, 5, 10, -7, 11], [1, 0, 2, -3, -2, -8, 8, 2, 3, -2], [-5, -2, -1, -8, -11, 11, 8, 1, 2, 5], [2, 1, 9, -4, 4, -9, -1, -2, 0, 5], [10, -12, 12, -10, 0, -4, 13, -5, 13, -4], [4, -1, -1, 6, 18, 6, -1, -1, 4, 0], [-4, 5, -14, 14, -5, 4, 0, 6, 3, 6], [-5, 5, 0, 7, -8, 0, 8, 0, -8, 7], [0, 5, -1, 4, -11, -5, -11, 4, -1, 5], [-7, -3, -12, -3, -7, 0, 3, 11, -11, -3], [-5, 0, 7, -9, 9, -9, 9, -9, 9, -7], [-1, 1, 0, 1, -1, 0, 10, 10, -10, -10], [0, -7, -4, -2, -5, 0, -5, -2, -4, -7], [-10, 20, -6, 13, -6, 20, -10, 0, -8, 8], [1, 9, 1, 0, 4, 7, -7, 7, -7, -4], [-6, 4, -4, 6, 0, 8, 1, 10, 1, 8], [-2, 11, -12, 13, -13, 12, -11, 2, 0, 21], [-5, 6, 3, 6, -5, 0, 4, 4, -4, -4], [-12, 5, -9, 13, -13, 9, -5, 12, 0, -2], [7, -3, 7, 0, 10, 2, 5, -5, -2, -10], [2, 9, -2, 3, 0, 3, -2, 9, 2, 0], [10, 1, -4, 4, -1, -10, 0, 3, 9, 3], [17, -10, 17, 0, 0, -2, 6, -6, 2, 0], [1, 1, 6, -4, 4, -6, -1, -1, 0, -14], [14, 0, 7, -13, 18, -11, 11, -18, 13, -7], [7, 9, -7, 9, 7, 0, 0, 1, -1, 0], [-15, 11, -11, 15, 0, 6, -9, 15, -9, 6], [-12, -5, -12, 0, -9, 8, -1, 1, -8, 9], [6, -6, 7, -16, 7, -16, 7, -6, 6, 0], [-16, 8, -9, 15, -9, 8, -16, 0, -10, 10], [-6, 6, 0, 4, 10, -2, -3, -2, 10, 4], [16, -8, 10, -8, 16, 0, 16, -13, 13, -16], [-12, -2, 3, -2, -12, 0, -14, -3, 3, 14], [0, -5, 8, 8, 0, 16, 0, 8, 8, -5], [13, -6, 5, -22, 5, -6, 13, 0, 5, -5], [-1, 12, 1, -1, -12, 1, 0, 7, -15, 7], [0, 2, 2, -12, -4, -12, 2, 2, 0, 0], [16, -2, 5, 6, 5, -2, 16, 0, 7, -7], [-2, 2, -2, 2, 0, -3, 9, -6, 9, -3], [13, -11, 21, -13, 24, -24, 13, -21, 11, -13], [3, 0, 7, -14, -3, 0, 0, 3, 14, -7], [-2, 2, 17, -2, 8, -8, 2, -17, -2, 2], [0, 3, 6, -15, 1, -1, 1, -15, 6, 3], [0, 2, -9, 9, -2, 0, 0, 5, 14, 5], [-7, -14, 5, -5, 14, 7, 0, -7, 8, -7], [-13, 15, -13, 0, -5, 8, 4, -4, -8, 5], [0, -21, 8, -9, 4, -7, 4, -9, 8, -21], [-6, 16, -6, 7, -3, 3, -7, 6, -16, 6], [-1, -1, -9, 8, -15, 15, -8, 9, 1, 1], [-9, 16, -9, 0, -21, 13, -14, 14, -13, 21], [19, -1, 2, 7, 2, -1, 19, 0, 8, -8], [9, -9, 9, -9, 0, -8, 2, -20, 2, -8], [-2, 10, 7, 10, -2, 0, -11, 14, -14, 11], [2, -8, 1, -8, 2, 0, -4, -8, 8, 4], [-9, 18, -18, 9, 0, 7, -17, 12, -17, 7], [17, -21, 23, -15, 23, -21, 17, 0, 13, -13], [20, -10, 19, -19, 10, -20, 0, -8, 16, -8], [-12, 0, -6, -5, 8, -5, -6, 0, -12, 0], [1, 12, -12, -1, 0, -11, 7, 11, 7, -11], [8, -5, -8, -5, 8, 0, 6, -13, 13, -6], [9, 0, 2, -8, -8, -1, 1, 8, 8, -2], [-1, -1, 0, 5, 8, -8, -5, 0, 1, 1], [-2, -5, -7, -2, 2, 7, 5, 2, 0, -1], [3, 4, 3, 0, 11, 15, -8, 8, -15, -11], [-1, 5, -10, 10, -5, 1, 0, 3, -24, 3], [-9, 11, -16, 3, -7, 3, -16, 11, -9, 0], [-15, 15, 0, -4, -15, 18, -19, 18, -15, -4], [-10, -7, -9, 6, -5, 5, -6, 9, 7, 10], [-1, 0, -2, -9, -1, 14, -14, 1, 9, 2], [-18, 0, -7, -1, 8, -8, 1, 7, 0, 18], [5, -21, 13, -6, 6, -13, 21, -5, 0, -1], [6, -11, 6, 0, -2, -2, -11, 11, 2, 2], [-23, 3, -7, 8, -7, 3, -23, 0, -3, 3], [-3, 7, 6, 20, 6, 7, -3, 0, 10, -10], [-8, 8, 0, -7, 14, 0, 0, 0, 14, -7], [0, 7, -3, 4, 7, -16, 7, 4, -3, 7], [7, 0, 10, -7, 0, -18, 18, 0, 7, -10], [6, -6, 6, -6, 0, -10, 12, -15, 12, -10], [-7, -4, 2, -2, 4, 7, 0, -3, -5, -3], [3, -17, 8, 4, 8, -17, 3, 0, -2, 2], [9, 14, 8, -3, 3, -8, -14, -9, 0, -10], [15, -1, -4, 4, 3, -3, -4, 4, 1, -15], [4, 9, 0, -7, -4, 4, 7, 0, -9, -4], [0, 27, -7, 24, -14, 12, -14, 24, -7, 27], [9, -24, 9, 0, 8, -16, 23, -23, 16, -8], [19, -14, 12, -7, 7, -7, 12, -14, 19, 0], [20, -11, 6, 2, 1, 2, 6, -11, 20, 0], [8, -8, 0, 19, 9, 1, 12, 1, 9, 19], [-13, 5, -16, 16, -16, 16, -5, 13, 0, 8], [2, 6, 5, 6, 2, 0, 14, 3, -3, -14], [-23, 5, -23, 0, 3, 0, -12, 12, 0, -3], [-12, 13, -19, 19, -13, 12, 0, 7, -21, 7], [-19, 18, -9, -16, -9, 18, -19, 0, -12, 12], [0, -16, 15, -1, 17, -10, 17, -1, 15, -16], [16, -26, 15, -26, 16, 0, 12, -21, 21, -12], [-3, 0, 2, 14, 1, -19, 19, -1, -14, -2], [-20, 0, -13, 14, -7, 8, -8, 7, -14, 13], [9, -14, 14, -9, 0, 7, -9, -21, -9, 7], [-16, 22, -14, 22, -17, 22, -14, 22, -16, 0], [1, -5, -20, 0, -20, -5, 1, 0, 11, -11], [11, 19, 11, 0, -13, 6, 1, -1, -6, 13], [-4, -9, 9, 4, 0, -4, 3, -16, 3, -4], [1, -3, -2, 4, 4, -4, -4, 2, 3, -1], [0, -11, 6, -9, 15, -19, 15, -9, 6, -11], [11, 8, -9, 9, -9, 9, -8, -11, 0, 10], [-17, 17, 0, 18, -18, 0, -2, 0, -18, 18], [16, -16, 0, -15, 21, -15, 15, -15, 21, -15], [-8, 4, 11, -1, 1, -11, -4, 8, 0, -21], [14, 0, 18, 2, -15, -2, 2, 15, -2, -18], [4, 0, 9, 3, -3, -9, 0, -4, 0, 15], [-6, 9, -2, 4, -2, 9, -6, 0, -19, 19], [-27, 17, -13, 22, -4, 22, -13, 17, -27, 0], [14, -20, 2, -2, 20, -14, 0, -22, 3, -22], [-3, 3, 0, -17, -5, -9, 6, -9, -5, -17], [-1, 0, -9, 1, -4, 19, -19, 4, -1, 9], [-17, 26, -3, 26, -17, 0, -10, -6, 6, 10], [-27, 3, -3, 27, 0, 9, 0, 21, 0, 9], [0, -7, 5, 0, 5, -7, 0, 0, -23, 23], [-14, 15, -38, 10, -20, 10, -38, 15, -14, 0], [-3, -1, -3, 0, 6, -11, 10, -10, 11, -6]]
